/** 
 *******************************************************************************
 * @file    main.c
 * @brief   RAMP Sample Application.
 * @version V1.0.1.0
 * $Date:: 2019-05-10 15:07:39 #$
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2019
 *******************************************************************************
 */
#ifdef __cplusplus
 extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include "TMPM4Ky.h"  /*!< TMPM4Ky Group Header file. */
#include "uart_io.h"
#include <stdio.h>
#include <string.h>
#include <stdlib.h>

/**
 *  @addtogroup Example
 *  @{
 */

/** 
 *  @defgroup RAMP RAMP Sample Appli
 *  @{
 */
/*------------------------------------------------------------------------------*/
/*  Macro Function                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup RAMP_Private_macro RAMP Private Macro
 *  @{
 */
/** 
 *  @defgroup LEDBlinkConfiguration LED Blink Time Configuration
 *  @brief    LED Blink Interval Time(Interrupt Cycle).
 *  @{
 */
#define CFG_LED_BLINK_TIME             ((uint32_t)(100))
/** 
 *  @}
 */ /* End of group LEDBlinkConfiguration */
 /** 
 *  @defgroup  GPIO_EN GPIO Enable/Disable
 *  @brief     GPIO Enable/Disable
 *  @{
 */
#define GPIO_DISABLE                    ((uint32_t)0x00000000)    /*!< Disable                 */
#define GPIO_ENABLE                     ((uint32_t)0x00000001)    /*!< Enable                  */
/** 
 *  @}
 */ /* End of group GPIO_EN */
 /** 
 *  @defgroup  FSYS_EN FSYS Enable/Disable
 *  @brief     FSYS Enable/Disable
 *  @{
 */
#define FSYS_DISABLE                    ((uint32_t)0x00000000)    /*!< Disable                 */
#define FSYS_ENABLE                     ((uint32_t)0x00000001)    /*!< Enable                  */
/** 
 *  @}
 */ /* End of group FSYS_EN */
 
/** 
 *  @defgroup DataLengtheConfiguration Data Length Configuration
 *  @brief    Data Length Configuration.
 *  @{
 */
#define CFG_DATA_LENGTH            ((uint32_t)20)
/** 
 *  @}
 */ /* End of group DataLengtheConfiguration */

 /** 
 *  @defgroup  ReceiveError Receive Error 
 *  @brief    Receive Error  .
 *  @{
 */
#define ERROR_UART_RECEIVE      ((uint32_t)(1))
#define ERROR_COMPARE           ((uint32_t)(3))
/** 
 *  @}
 */ /* End of group ReceiveError */

/** 
 *  @defgroup RAMPCTL_RAMPF RAMPF
 *  @brief    [Bit:1]: PARITY Error Interrupt Control.
 *  @{
 */
#define RAMP_RAMPCTL_RAMPF_MASK          ((uint32_t)0x00000002)        /*!< for Bit Mask.  */
#define RAMP_RAMPCTL_RAMPF_DISABLE       ((uint32_t)0x00000000)        /*!< Error Interrupt Disable.       */
#define RAMP_RAMPCTL_RAMPF_ENABLE        ((uint32_t)0x00000002)        /*!< Error Interrupt Enable.        */
/**
 *  @}
 */ /* End of group RAMPCTL_RAMPF */

/** 
 *  @defgroup RAMPCTL_RAMPEN RAMPEN
 *  @brief    [Bit:1]: PARITY Error Detection Control.
 *  @{
 */
#define RAMP_RAMPCTL_RAMPEN_MASK         ((uint32_t)0x00000001)        /*!< for Bit Mask.  */
#define RAMP_RAMPCTL_RAMPEN_DISABLE      ((uint32_t)0x00000000)        /*!< Error Detection Disable.       */
#define RAMP_RAMPCTL_RAMPEN_ENABLE       ((uint32_t)0x00000001)        /*!< Error Detection Enable.        */
/**
 *  @}
 */ /* End of group RAMPCTL_RAMPEN */
/** 
 *  @}
 */ /* End of group RAMP_Private_macro */

/*------------------------------------------------------------------------------*/
/*  Private Member                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup RAMP_Private_variables RAMP Private Variables
 *  @{
 */
static uint32_t parity_error = 0;           /*!< parity error               */
static uint32_t err_addr     = 0;           /*!< error address              */
static uint32_t error        = 0;           /*!< error flag.                */
static uint32_t cmp_flag     = 0;           /*!< Receive complete flag.     */
static uint8_t rx_data[CFG_DATA_LENGTH];    /*!< Read data buffer.          */
static uint8_t rx_length    = 6;            /*!< Compare Char length.       */
static uint8_t writestring[CFG_DATA_LENGTH] = {0};  /*!< Write data         */
static uint32_t counter  = CFG_LED_BLINK_TIME;      /*!< Blink Time Counter */
static const uint8_t cmpstring[6] = "write";       /*!< write command      */

#if defined ( __CC_ARM   )      /* RealView Compiler */
static uint8_t ram1[CFG_DATA_LENGTH] __attribute__((section("rw_iram2")));
#elif defined ( __ICCARM__ )    /*IAR Compiler */
static uint8_t ram1[CFG_DATA_LENGTH] @"rw_iram2";
#endif
/**
 *  @}
 */ /* End of group RAMP_Private_variables */

/*------------------------------------------------------------------------------*/
/*  Private Function                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup RAMP_Private_fuctions RAMP Private Fuctions
 *  @{
 */
static void GPIO_Config(void);
static void RAMP_Config(void);
static void ram_clear(void);
/*--------------------------------------------------*/
/** 
  * @brief  GPIO_Configure
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void GPIO_Config(void)
{
  /* (1) Enable the peripheral clock of PORT J */
  TSB_CG_FSYSMENA_IPMENA08 = FSYS_ENABLE;
  /* (2) Turn off PORT J pin 0 (LED0) */
  TSB_PJ_DATA_PJ0 = GPIO_DISABLE;
  /* (3) Select output mode on PORT J pin 0 (LED0) */
  TSB_PJ_CR_PJ0C = GPIO_ENABLE;
  /* (4) Turn off PORT J pin 1 (LED1) */
  TSB_PJ_DATA_PJ1 = GPIO_DISABLE;
  /* (5) Select output mode on PORT J pin 1 (LED1) */
  TSB_PJ_CR_PJ1C = GPIO_ENABLE;
  /* (6) Enable the peripheral clock of PORT A */
  TSB_CG_FSYSMENA_IPMENA00 = FSYS_ENABLE;
  /* (7) Select output mode on PORT A pin 0 (SWS) */
  TSB_PA_CR_PA0C = GPIO_ENABLE;
  TSB_PA_DATA_PA0 = GPIO_ENABLE;   /*SW select 1:SW7-4 */
  /* (8) Select input mode on PORT A pin 2 (SW4) */
  TSB_PA_IE_PA2IE = GPIO_ENABLE;
}
/*--------------------------------------------------*/
/** 
  * @brief  RAMP_Configure
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void RAMP_Config(void)
{
  uint32_t i;
  /* (1) Enable the peripheral clock of RAMP */
  TSB_CG_FSYSENA_IPENA01 = FSYS_ENABLE;
  /* (2) Enable the RAM Parity Control */
  TSB_RPAR0_CTL_RPAREN = RAMP_RAMPCTL_RAMPEN_ENABLE;
    for(i=0; i<CFG_DATA_LENGTH; i++)
    {
      ram1[i] = 0;
    }
    TSB_RPAR0->CTL = RAMP_RAMPCTL_RAMPF_ENABLE | RAMP_RAMPCTL_RAMPEN_ENABLE;;
    NVIC_EnableIRQ(INTPARI0_IRQn);
}
/*--------------------------------------------------*/
/** 
  * @brief  ram_clear
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void ram_clear(void)
{
    uint32_t i;
    for(i=0; i<CFG_DATA_LENGTH; i++)
    {
        rx_data[i] = 0;
        ram1[i] = 0;
        writestring[i] = 0;
    }
    rx_length = 0;
}

/**
 *  @}
 */ /* End of group RAMP_Private_fuctions */

/*------------------------------------------------------------------------------*/
/*  Public Function                                                             */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup RAMP_Exported_functions RAMP Exported Functions
 *  @{
 */
 
/*--------------------------------------------------*/
/** 
  * @brief  Main Function.
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
int main(void)
{
  uint32_t period;    /* Period for Systick */
  /*------------------------*/
  /* Port Setting           */
  /*------------------------*/
  GPIO_Config();
  uart_io_initialize();
  /* Period for 10msec SYSTICK */
  SystemCoreClockUpdate();
  period = SystemCoreClock / 100;
  if (SysTick_Config(period ))    /* systick interrupt cycle setting & interrupt enable*/
  {
  }
  /*------------------------*/
  /* RAMP Setting           */
  /*------------------------*/
  RAMP_Config();
  /*------------------------*/
  /*          Loop                         */
  /*------------------------*/ 
  while(1)
  {
    error = 0;
    cmp_flag = 0;
    ram_clear();
    (void)printf("\nCommand >");
    
    while(cmp_flag == 0)
    {
        int data;
        
        data = getchar();
        if ((data != -1) && (rx_length < CFG_DATA_LENGTH))
        {
            rx_data[rx_length] = (char)data;
            rx_length++;
            switch ((char)data)
            {
            /*--- CR: carriage return ---*/
            case '\r':
                /* no process */
                break;
            /*--- LF: line feed ---*/
            case '\n':
                cmp_flag = 1;
                break;
            /*--- default ---*/
            default:
                /* no process */
                break;
            }
        }
        else
        {
            cmp_flag = 1;
            error = ERROR_UART_RECEIVE;    /* Report an error */
        }
    }
    cmp_flag = 0;
    /*------------------------*/
    /* Compare Receiving Data */
    /*------------------------*/  
    if(error == 0)
    {
        if ( strncmp((char *)cmpstring, (char *)rx_data, (sizeof(cmpstring) - 1)) == 0 )
        {
            /* RAM AREA writing */
            uint32_t    i;
            uint32_t    j = 0;
            uint32_t    len = 0;
            
            if(rx_length > 2){
                len = rx_length - 2;
            }
            for(i=sizeof(cmpstring); i< len; ++i)
            {
                writestring[j] = rx_data[i];
                ++j;
            }
            if(strncpy((char *)ram1, (char const *)writestring, j) == NULL)
            {
                
            }
            if(strncmp((char *)ram1, (char *)writestring, j) != 0)
            {
                error = ERROR_COMPARE;
            }
        }
        else
        {
            error = ERROR_UART_RECEIVE;    /* Report an error */
        }
    }
    if(error == 0)
    {
        /* start USART transmission */
        (void)printf("write >%s\n",(const char *)writestring);
        (void)printf("read  >%s\n",(const char *)ram1);
    }
    else if(error == ERROR_UART_RECEIVE)
    {
        (void)printf("UART Input Error\n");
    }
    else if(error == ERROR_COMPARE)
    {
        (void)printf("Compare Error\n");
    }
    if(parity_error !=  0)
    {
        (void)printf("\nParity Error :Address = 0x%x\n", err_addr);
    }
  }
}
/*--------------------------------------------------*/
/** 
  * @brief  SysTick Handler
  * @param  -
  * @return -
  * @retval -
  */
/*--------------------------------------------------*/
void SysTick_Handler(void)
{
    if(parity_error == 1)
    {
        if (counter-- == 0) 
        {
            /* The end of the counting down period, the data is
            inverted and stored in a register PJ0           */
            TSB_PJ_DATA_PJ0 ^= 1;     /* led0 blink for 1sec period */
            counter = CFG_LED_BLINK_TIME;
        }
        else{
            /* no processing */
        }
    }
}

/*--------------------------------------------------*/
/** 
  * @brief  INTPARI0 Handler
  * @param  -
  * @return -
  * @retval -
  */
/*--------------------------------------------------*/
void INTPARI0_IRQHandler(void)
{
    uint32_t rparst = TSB_RPAR0->ST;
     
    if((rparst  & 0x01)== 0x01)
    {
        err_addr = TSB_RPAR0->EAD0;
    }
    else if((rparst & 0x02) == 0x02)
    {
        err_addr = TSB_RPAR0->EAD1;
    }
    
    TSB_RPAR0->CLR = rparst;
    parity_error = 1;
}

/**
 *  @}
 */ /* End of group RAMP_Exported_functions */

/**
 *  @}
 */ /* End of group RAMP */

/**
 *  @} 
 */ /* End of group Example */

#ifdef __cplusplus
}
#endif /* __cplusplus */
