/** 
 *******************************************************************************
 * @file    main.c
 * @brief   TRMOSC Sample Application.
 * @version V1.0.2.0
 * $Date:: 2019-06-25 11:52:33 #$
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2019
 *******************************************************************************
 */

#ifdef __cplusplus
 extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include <stdio.h>
#include "txz_sample_def.h"
#include "IOSCadjustment.h"
#include "trm_t32a.h"
#include "uart_io.h"

/**
 *  @addtogroup Example
 *  @{
 */

/** 
 *  @defgroup TRMOSC TRMOSC Sample Appli
 *  @{
 */

/*------------------------------------------------------------------------------*/
/*  Configuration                                                               */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup TRMOSC_Private_define TRMOSC Private Define
 *  @{
 */

/** 
 *  @defgroup ELOSCConfiguration ELOSC Configuration
 *  @brief    ELOSC Configuration.
 *  @{
 */
#define ELOSC_CFG_WARM_UP_TIME  ((uint32_t)(5000))      /*!< Warm up time(us). */
#define ELOSC_CFG_CLOCK         ((uint32_t)(32768))     /*!< Clock(hz).        */
/** 
 *  @}
 */ /* End of group ELOSCConfiguration */

/** 
 *  @defgroup  CGWUPLCR  CGWUPLCR Register
 *  @brief     LOSC warming up Register.
 *  @details   Detail.
 *  | Bit   | Bit Symbol |
 *  | :---  | :---       |
 *  | 31:27 | -          |
 *  | 26:12 | WUPTL      |
 *  | 11:8  | WUPTL      |
 *  | 7:2   | -          |
 *  | 1     | WULEF      |
 *  | 0     | WULON      |
 *  @{
 */
/* WUPTL  */
#define CGWUPLCR_WUPTL_HIGH_MASK        ((uint32_t)0x07FFF000)    /*!< WUPTL   :High Bit Mask.  */
/* WULEF  */
#define CGWUPLCR_WULEF_MASK             ((uint32_t)0x00000002)    /*!< WULEF   :Mask.           */
#define CGWUPLCR_WULEF_R_DONE           ((uint32_t)0x00000000)    /*!< WULEF   :[R]  :Done      */
#define CGWUPLCR_WULEF_R_RUNNING        ((uint32_t)0x00000002)    /*!< WULEF   :[R]  :Running   */
/* WULON  */
#define CGWUPLCR_WULON_MASK             ((uint32_t)0x00000001)    /*!< WULON   :Mask.           */
#define CGWUPLCR_WULON_W_ENABLE         ((uint32_t)0x00000001)    /*!< WULON   :[W]  :Enable     */
/** 
 *  @}
 */ /* End of group CGWUPLCR */

/** 
 *  @defgroup  RLMLOSCCR  RLMLOSCCR Register
 *  @brief     LOSC Control Register.
 *  @details   Detail.
 *  | Bit   | Bit Symbol |
 *  | :---  | :---       |
 *  | 31:1  | -          |
 *  | 0     | XTEN       |
 *  @{
 */
/* XTEN */
#define RLMLOSCCR_XTEN_RW_DISABLE       ((uint32_t)0x00000000)    /*!< XTEN :[R/W]  :Disable  */
#define RLMLOSCCR_XTEN_RW_ENABLE        ((uint32_t)0x00000001)    /*!< XTEN :[R/W]  :Enable   */
/** 
 *  @}
 */ /* End of group RLMLOSCCR */

/** 
 *  @}
 */ /* End of group TRMOSC_Private_define */


/*------------------------------------------------------------------------------*/
/*  Macro Definition                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup TRMOSC_Private_define TRMOSC Private Define
 *  @{
 */
#define COUNTMAX               (10)            /*!< retry counter max value1 */
#define MAIN_NULL       ((void *)0)     /*!< NULL. */
#define USER_ID_REMOTE  ((uint32_t)0)   /*!< User ID :Remote Control (Any value) */
#define USER_ID_UART    ((uint32_t)1)   /*!< User ID :USB UART (Any value) */


/** 
 *  @}
 */ /* End of group TRMOSC_Private_define */


/*------------------------------------------------------------------------------*/
/*  Enumerated Type Definition                                                  */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup TRMOSC_Private_define TRMOSC Private Define
 *  @{
 */
 /** 
 *  @defgroup  GPIO_EN GPIO Enable/Disable
 *  @brief     GPIO Enable/Disable
 *  @{
 */
#define GPIO_DISABLE                    ((uint32_t)0x00000000)    /*!< Disable                 */
#define GPIO_ENABLE                     ((uint32_t)0x00000001)    /*!< Enable                  */
/** 
 *  @}
 */ /* End of group GPIO_EN */
 /** 
 *  @defgroup  FSYS_EN FSYS Enable/Disable
 *  @brief     FSYS Enable/Disable
 *  @{
 */
#define FSYS_DISABLE                    ((uint32_t)0x00000000)    /*!< Disable                 */
#define FSYS_ENABLE                     ((uint32_t)0x00000001)    /*!< Enable                  */
/** 
 *  @}
 */ /* End of group FSYS_EN */

/** 
 *  @}
 */ /* End of group TRMOSC_Private_define */

/*------------------------------------------------------------------------------*/
/*  Private Member                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup TRMOSC_Private_variables TRMOSC Private Variables
 *  @{
 */
static uint32_t RetryCount=0;
static int32_t  AdjustResult;

/**
 *  @}
 */ /* End of group TRMOSC_Private_variables */


/*------------------------------------------------------------------------------*/
/*  Private Function                                                            */
/*------------------------------------------------------------------------------*/
static void InitGpio(void);
/** 
 *  @defgroup TRMOSC_Private_fuctions TRMOSC Private Fuctions
 *  @{
 */

static void InitGpio(void)
{
    /* (1) Enable the peripheral clock of PORT E */
    TSB_CG_FSYSMENA_IPMENA04 = FSYS_ENABLE;
    /* (2) Select function mode (T32A03OUTA) on PORT E pin 2    */
    TSB_PE_FR4_PE2F4 = GPIO_ENABLE;
    /* (3) Select output mode on PORT E pin 2    */
    TSB_PE_CR_PE2C = GPIO_ENABLE;
    /* (4) Enable the peripheral clock of PORT P */
    TSB_CG_FSYSMENA_IPMENA13 = FSYS_ENABLE;
    /* (5) Select Input mode on PORT P pin 3    */
    TSB_PP_IE_PP3IE = GPIO_ENABLE;
    /* (6) Select function mode (T32A01INA0) on PORT P pin 3    */
    TSB_PP_FR4_PP3F4 = GPIO_ENABLE;
    /* (7) Enable the peripheral clock of T32A01 */
    TSB_CG_FSYSMENA_IPMENA29 = FSYS_ENABLE;
    /* (8) Enable the peripheral clock of T32A03 */
    TSB_CG_FSYSMENA_IPMENA31 = FSYS_ENABLE;
    /* (9) Enable the peripheral clock of TRM */
    TSB_CG_FSYSMENB_IPMENB15 = FSYS_ENABLE;
}
/**
 *  @}
 */ /* End of group TRMOSC_Private_fuctions */


/*------------------------------------------------------------------------------*/
/*  Public Function                                                             */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup TRMOSC_Exported_functions TRMOSC Exported Functions
 *  @{
 */

/*--------------------------------------------------*/
/** 
  * @brief  The main function of TRMOSC demo
  * @param  -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
int main(void)
{
    uint32_t tmp, i;

    InitGpio();
    /*------------------------*/
    /* UART Setting            */
    /*------------------------*/
    uart_io_initialize();
    InitT32A_Sout();
    InitT32A_RefIn();
    InitTRM();

    if(printf("%s\r\n", "---------------") < 0){ /* Do Nothing */ };
    if(printf("%s\r\n", "| TRMOSC_demo |") < 0){ /* Do Nothing */ };
    if(printf("%s\r\n", "---------------") < 0){ /* Do Nothing */ };

    if(printf("%s\r\n", "please wait...") < 0){ /* Do Nothing */ };
    for(i=0; i<0x00400000; i++){};
    if(printf("%s\r\n", "---------------") < 0){ /* Do Nothing */ };
    if(printf("%s\r\n", "|    start    |") < 0){ /* Do Nothing */ };
    if(printf("%s\r\n", "---------------") < 0){ /* Do Nothing */ };

    for(RetryCount=0; RetryCount <= COUNTMAX; RetryCount++) 
    {
        AdjustResult = IOSCAdjustment();
        if (AdjustResult == TRMOSC_DONE) 
        {
            if(printf("%s\r\n", "TRMOSC_DONE") < 0){ /* Do Nothing */ };
            if(GetTrimmingControlValue() == TRMOSC_CONTROL_ENABLE) 
            {
                tmp = GetCoarseTrimValueSet();
                if(printf("%s%X\r\n", "<TRIMSETC>:", tmp) < 0){ /* Do Nothing */ };
                tmp = GetFineTrimValueSet();
                if(printf("%s%X\r\n", "<TRIMSETF>:", tmp) < 0){ /* Do Nothing */ };
            }
            else 
            {
                if(printf("%s\r\n", "Inital trimming Value") < 0){ /* Do Nothing */ };
            }
            break;
        }
        else if (AdjustResult == TRMOSC_RUN) 
        {
            if(printf("%s\r\n", "TRMOSC_RUN") < 0){ /* Do Nothing */ };
            if(GetTrimmingControlValue() == TRMOSC_CONTROL_ENABLE) 
            {
                tmp = GetCoarseTrimValueSet();
                if(printf("%s%X\r\n", "<TRIMSETC>:", tmp) < 0){ /* Do Nothing */ };
                tmp = GetFineTrimValueSet();
                if(printf("%s%X\r\n", "<TRIMSETF>:", tmp) < 0){ /* Do Nothing */ };
            }
            else 
            {
                if(printf("%s\r\n", "Inital trimming Value") < 0){ /* Do Nothing */ };
            }
        }
        else 
        {
            if(printf("%s\r\n", "TRMOSC_ERROR") < 0){ /* Do Nothing */ };
        }
    }

    while(1){};
}

#ifdef DEBUG
/*--------------------------------------------------*/
/** 
  * @brief  Failure Assert.
  * @note   for debug
  */
/*--------------------------------------------------*/
void assert_failed(char *file, int32_t line)
{
    while (1) {
        __NOP();
    }
}
#endif
/**
 *  @}
 */ /* End of group TRMOSC_Exported_functions */

/**
 *  @}
 */ /* End of group TRMOSC */

/**
 *  @} 
 */ /* End of group Example */

#ifdef __cplusplus
}
#endif /* __cplusplus */
